#!/usr/bin/env python3
# vim: set shiftwidth=4 softtabstop=4:
"""Script to run test with local modules
"""
import logging
import os
import sys
from configparser import ConfigParser
from subprocess import call

from psycopg2 import OperationalError, connect

from odoo_scripts.docker_postgresql import docker_run_postgresql

from .config import Config
from .docker_dev_start import main as docker_dev_start_main
from .docker_flake8 import apply_flake8, parser_add_flake8_group
from .docker_isort import apply_isort, parser_add_isort_group
from .docker_postgresql import POSTGRES_PASSWORD
from .parsing import apply, basic_parser

_logger = logging.getLogger(__name__)

__version__ = "3.0.0"
__date__ = "2018-04-13"
__updated__ = "2021-10-21"


def main(argv=None):  # IGNORE:C0111
    """Parse arguments and docker build"""
    program_version = __version__
    program_build_date = str(__updated__)
    program_version_message = "%%(prog)s %s (%s)" % (
        program_version,
        program_build_date,
    )
    program_shortdesc = __doc__.split(".")[0]
    program_license = """%s

  Created by Vincent Hatakeyama on %s.
  Copyright 2018, 2020 XCG Consulting. All rights reserved.

  Licensed under the MIT License

  Distributed on an "AS IS" basis without warranties
  or conditions of any kind, either express or implied.

USAGE
""" % (
        program_shortdesc,
        str(__date__),
    )

    # the script assume it is launched from the super project
    project_path = os.path.realpath(".")
    project_name = os.path.basename(project_path)

    # TODO add a way to store configuration options in a project file
    # Argument parsing
    parser = basic_parser(
        description=program_license, version=program_version_message
    )
    docker_group = parser.add_mutually_exclusive_group()
    docker_group.add_argument(
        "--docker",
        help="Use docker to launch tests",
        action="store_true",
        default=True,
        dest="docker",
    )
    docker_group.add_argument(
        "--no-docker",
        help="Do not use docker to launch tests",
        action="store_false",
        dest="docker",
    )
    parser.add_argument(
        "-d",
        "--dbname",
        help="Database name [default: %(default)s]",
        default="%s_test" % project_name,
    )
    parser.add_argument(
        "--db_user",
        help="Odoo database user [default: %(default)s]",
        default=None,
    )
    parser.add_argument(
        "--db_password",
        help="Odoo database password [default: %(default)s]",
        default=None,
    )
    odoo_log_levels = ["info", "warn", "debug"]  # TODO there are more
    parser.add_argument(
        "--log-level",
        help="Override odoo log level (for tests and install)",
        default=None,
        choices=odoo_log_levels,
    )
    parser.add_argument(
        "--install-log-level",
        help="Override odoo log level (for install)",
        default="warn",
        choices=odoo_log_levels,
    )
    parser.add_argument(
        "--test-log-level",
        help="Override odoo log level (for test)",
        default=None,
        choices=odoo_log_levels,
    )
    parser.add_argument(
        "--no-create-database", dest="createdb", action="store_false"
    )
    parser.add_argument(
        "-i",
        "--install",
        help="Modules to install (override defaults from setup.cfg)",
        default=None,
    )
    parser.add_argument(
        "--test",
        help="Modules to test (override defaults from setup.cfg)",
        default=None,
    )
    parser.add_argument(
        "-p",
        "--dbport",
        help="Database port [default: %(default)s]",
        default=None,
    )
    parser.add_argument(
        "--start-postgresql",
        help="Start a Postgresql docker [default: %(default)s]",
        action="store_true",
    )
    parser.add_argument(
        "--log-handler",
        help="log handler [default: %(default)s]",
        default=None,
        action="append",
    )
    parser.add_argument(
        "--load-language",
        help="specifies the languages for the translations you want to be"
        " loaded",
        default=None,
        dest="LOAD_LANGUAGE",
    )
    parser_add_flake8_group(parser)
    parser_add_isort_group(parser)
    # TODO options
    # - db host/uri (include socket)
    # - db user for creation/remove

    nmspc = parser.parse_args(argv)
    apply(nmspc)
    dbname = nmspc.dbname
    odoo_db_user = nmspc.db_user
    odoo_db_password = nmspc.db_password
    recreate_db = nmspc.createdb
    install_log_level = (
        nmspc.log_level if nmspc.log_level else nmspc.install_log_level
    )
    test_log_level = (
        nmspc.log_level if nmspc.log_level else nmspc.test_log_level
    )
    override_installed_module = nmspc.install
    override_tested_module = nmspc.test
    dbport = nmspc.dbport
    start_postgresql = nmspc.start_postgresql

    # Get parameters from setup file

    config = Config()

    registry = config.registry
    project = config.image
    languages = config.load_language
    postgresql_version = config.postgresql_version
    odoo_type = config.odoo_type
    # TODO factorize with docker_dev_start
    image = "%s/%s:latest" % (registry, project)
    _logger.debug("Docker image: %s", image)
    extensions = config.pg_extensions

    # read from odoo.conf if it exists

    sample_conf = "conf/dev/odoo.conf"
    if os.path.exists(sample_conf):
        _logger.info("Reading from sample configuration %s", sample_conf)
        c = ConfigParser()
        c.read(sample_conf)
        if not odoo_db_user:
            if c.has_option("options", "db_user"):
                odoo_db_user = c.get("options", "db_user")
        if not odoo_db_password:
            if c.has_option("options", "db_password"):
                odoo_db_password = c.get("options", "db_password")
        if c.has_option("options", "unaccent") and c.getboolean(
            "options", "unaccent"
        ):
            extensions.append("unaccent")
    else:
        _logger.debug("No sample configuration %s", sample_conf)
        extensions.append("unaccent")

    if not odoo_db_user:
        if start_postgresql:
            odoo_db_user = "odoo"
        else:
            if not odoo_db_user and config.db_user is not None:
                odoo_db_user = config.db_user
            if not odoo_db_user:
                _logger.warning("No database user found or given")
    if not odoo_db_password:
        if start_postgresql:
            odoo_db_password = "odoo"
        else:
            if not odoo_db_password and config.db_password:
                odoo_db_password = config.db_password
            if not odoo_db_password:
                _logger.warning("No database password found or given")

    # Do stuff

    if nmspc.docker:
        # TODO detect that user is member of docker group
        args = [
            "-d",
            dbname,
            "--stop-after-init",
            "--max-cron-threads=0",
            "--no-flake8",
            "--no-isort",
            "--no-dev",
        ]
        if odoo_db_user:
            args.append("--db_user")
            args.append(odoo_db_user)
        if odoo_db_password:
            args.append("--db_password")
            args.append(odoo_db_password)
        if dbport:
            args.append("--dbport")
            args.append(dbport)

    apply_flake8(nmspc, odoo_type)
    apply_isort(nmspc)
    if nmspc.log_handler:
        for lh in nmspc.log_handler:
            args.append("--log-handler")
            args.append(lh)
    if start_postgresql:
        args.append("--start-postgresql")
    if nmspc.LOAD_LANGUAGE:
        args.append("--load-language")
        args.append(nmspc.LOAD_LANGUAGE)
    if recreate_db:
        if start_postgresql:
            container, stop_method, socket_path = docker_run_postgresql(
                project_name, postgresql_version
            )
            # system user used in the pg docker image
            pg_user = "postgres"
            try:
                if start_postgresql:
                    connect(
                        user=odoo_db_user,
                        password=odoo_db_password,
                        database="postgres",
                        host=socket_path,
                        # need to be set to avoid using value from an ENV VAR
                        port=5432,
                    )
            except OperationalError:
                connection = connect(
                    user=pg_user,
                    database="postgres",
                    host=socket_path,
                    port=5432,
                    password=POSTGRES_PASSWORD,
                )
                _logger.info("Creating role %s", odoo_db_user)
                with connection.cursor() as cursor:
                    # not injection safe but you are on your own machine
                    # with already full access to db
                    cursor.execute(
                        "CREATE ROLE %s LOGIN CREATEDB PASSWORD %%s"
                        % odoo_db_user,
                        (odoo_db_password,),
                    )
                connection.commit()
                connection.close()
            odoo_connection = connect(
                user=pg_user, database="postgres", host=socket_path, port=5432
            )
            odoo_connection.autocommit = True
            with odoo_connection.cursor() as cursor:
                _logger.debug("Drop database %s", dbname)
                cursor.execute('DROP DATABASE IF EXISTS "%s"' % dbname)
                _logger.debug("Create database %s", dbname)
                cursor.execute(
                    'CREATE DATABASE "%s" OWNER %s' % (dbname, odoo_db_user)
                )
            odoo_connection = connect(
                user=pg_user, database=dbname, host=socket_path, port=5432
            )
            odoo_connection.autocommit = True
            with odoo_connection.cursor() as cursor:
                for extension in extensions:
                    _logger.info("Adding extension %s", extension)
                    cursor.execute("CREATE EXTENSION %s" % extension)
            # TODO do we really need to stop the container each time?
            #  that seems faster
            # stop_method()
        else:
            # TODO use psycopg2 (see above)
            # drop database
            _logger.info("Drop any existing database %s", dbname)
            dropdb = ["dropdb", dbname, "--if-exists", "--no-password"]
            if dbport:
                dropdb.append("-p")
                dropdb.append(dbport)
            result = call(dropdb)
            if result:
                return result
            # create database
            _logger.info("Create database %s", dbname)
            create_db = ["createdb", dbname, "--no-password"]
            if odoo_db_user:
                create_db.append("-O")
                create_db.append(odoo_db_user)
            if dbport:
                create_db.append("-p")
                create_db.append(dbport)
            result = call(create_db)
            if result:
                return result
            # add unaccent if needed
            for extension in extensions:
                _logger.info("Adding extension %s", extension)
                create_extension = [
                    "psql",
                    dbname,
                    "-c",
                    "CREATE EXTENSION %s;" % extension,
                ]
                if dbport:
                    create_extension.append("-p")
                    create_extension.append(dbport)
                result = call(create_extension)
                if result:
                    return result

        # TODO start odoo and detect if install fails
        if nmspc.docker:
            install_args = list(args)
            if override_installed_module:
                install_args.append("--install")
                install_args.append(override_installed_module)
            else:
                install_args.append("--install-default")
            if languages:
                install_args.append("--load-language")
                install_args.append(languages)
            if install_log_level:
                install_args.append("--log-level")
                install_args.append(install_log_level)
            result = docker_dev_start_main(install_args)

            if result:
                return result
        else:
            raise NotImplementedError
    # start odoo and detect if test fails
    if nmspc.docker:
        test_args = list(args)
        if override_tested_module:
            test_args.append("--test")
            test_args.append(override_tested_module)
        else:
            test_args.append("--test-default")
        if test_log_level:
            test_args.append("--log-level")
            test_args.append(test_log_level)
        return docker_dev_start_main(test_args)
    else:
        raise NotImplementedError
    return 255


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))
