#!/usr/bin/env python3
# vim: set shiftwidth=4 softtabstop=4:
"""Run flake8 in a docker
"""
import argparse
import atexit
import logging
import os
import sys

from .config import Config
from .docker_client import DockerClient
from .parsing import apply, basic_parser

__version__ = "1.0.0"
__date__ = "2020-09-10"
__updated__ = "2020-09-10"

_logger = logging.getLogger(__name__)


def __parser():
    program_version = __version__
    program_build_date = str(__updated__)
    program_version_message = "%%(prog)s %s (%s)" % (
        program_version,
        program_build_date,
    )
    program_shortdesc = __doc__.split(".")[0]
    program_license = """%s

      Created by Vincent Hatakeyama on %s.
      Copyright 2020 XCG Consulting. All rights reserved.

      Licensed under the MIT License

      Distributed on an "AS IS" basis without warranties
      or conditions of any kind, either express or implied.

    USAGE
    """ % (
        program_shortdesc,
        str(__date__),
    )
    parser = basic_parser(program_license, program_version_message)
    return parser


def parser_add_flake8_group(parser: argparse.ArgumentParser):
    """Add flake8 option (--flake8/--no-flake8) to the given parser.
    Defaults to not using flake8."""
    flake8_group = parser.add_mutually_exclusive_group()
    flake8_group.add_argument(
        "--flake8",
        help="Run flake8",
        action="store_true",
        dest="flake8",
        default=False,
    )
    flake8_group.add_argument(
        "--no-flake8",
        help="Do not run flake8 [default]",
        action="store_false",
        dest="flake8",
        default=False,
    )


def apply_flake8(namespace: argparse.Namespace, odoo_type: str):
    """Run flake8 if the option was set."""
    if namespace.flake8:
        flake8(odoo_type)


def main(argv=None):
    """Copy modules for a build, callable version that parses arguments"""
    parser = __parser()
    nmspc = parser.parse_args(argv)
    apply(nmspc)
    c = Config()
    odoo_type = c.odoo_type
    return flake8(odoo_type)


def flake8(odoo_type: str):
    """Run flake8"""
    client = DockerClient.client
    repository = "xcgd/flake8"
    if odoo_type in ("odoo7", "odoo8", "odoo9", "odoo10"):
        tag = "2"
        _logger.info("Running flake8 for Python 2")
    else:
        tag = "3"
        _logger.info("Running flake8 for Python 3")
    client.images.pull(repository, tag)
    image = "{}:{}".format(repository, tag)
    container = None

    # XXX When pressing ctrl-c during the container creating, the container
    #  is not removed as there is no return of the value of the Container
    #  object. There is several ways to do that:
    #  - give the container a unique name we know, and use that to clean
    #  - maybe use some kind of time frame to identify the container
    #  - warn the user of a potential container leak by using signals

    def stop_remove():
        """Stop then remove the container."""
        if container:
            if container.status == "running":
                container.stop()
                container.wait()
            container.remove()

    atexit.register(stop_remove)

    # split create and start so in case of SIGTERM, container has fewer chances
    # of being None
    container = client.containers.create(
        image,
        command=["."],
        volumes={os.environ["PWD"]: {"bind": "/mnt", "mode": "ro"}},
        working_dir="/mnt",
        detach=True,
    )
    container.start()
    result = container.wait()
    atexit.unregister(stop_remove)
    atexit.register(container.remove)
    print(container.logs().decode().rstrip())
    if result.get("Error", None):
        _logger.warning(result.decode())
    container.remove(v=True)
    atexit.unregister(container.remove)
    return result["StatusCode"]


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))
