#!/usr/bin/env python3
# vim: set shiftwidth=4 softtabstop=4:
"""Run isort in a docker
"""
import argparse
import atexit
import logging
import os
import sys

import docker

from .docker_client import DockerClient
from .parsing import apply, basic_parser

__version__ = "1.0.1"
__date__ = "2020-09-10"
__updated__ = "2021-10-21"

_logger = logging.getLogger(__name__)


def __parser():
    program_version = __version__
    program_build_date = str(__updated__)
    program_version_message = "%%(prog)s %s (%s)" % (
        program_version,
        program_build_date,
    )
    program_shortdesc = __doc__.split(".")[0]
    program_license = """%s

      Created by Vincent Hatakeyama on %s.
      Copyright 2020 XCG Consulting. All rights reserved.

      Licensed under the MIT License

      Distributed on an "AS IS" basis without warranties
      or conditions of any kind, either express or implied.

    USAGE
    """ % (
        program_shortdesc,
        str(__date__),
    )
    parser = basic_parser(program_license, program_version_message)
    return parser


def parser_add_isort_group(parser: argparse.ArgumentParser):
    """Add isort option (--isort/--no-isort) to the given parser.
    Defaults to not using isort."""
    isort_group = parser.add_mutually_exclusive_group()
    isort_group.add_argument(
        "--isort",
        help="Run isort",
        action="store_true",
        dest="isort",
        default=False,
    )
    isort_group.add_argument(
        "--no-isort",
        help="Do not run isort [default]",
        action="store_false",
        dest="isort",
        default=False,
    )


def apply_isort(namespace: argparse.Namespace):
    """Run isort if the option was set."""
    if namespace.isort:
        isort()


def main(argv=None):
    """Copy modules for a build, callable version that parses arguments"""
    parser = __parser()
    nmspc = parser.parse_args(argv)
    apply(nmspc)
    isort()
    return 0


def isort(pull=True):
    """Run isort"""
    client = DockerClient.client
    if pull:
        _logger.info("Pulling isort")
        # TODO find a better way to avoid error when pulling without a
        #  connection
        try:
            client.images.pull(repository="xcgd/isort", tag="odoo")
        except docker.errors.APIError:
            _logger.warning(
                "No connection to server, using existing version, if any"
            )
    _logger.info("Running isort for Odoo")
    container = None

    # XXX When pressing ctrl-c during the container creating, the container
    #  is not removed as there is no return of the value of the Container
    #  object. There is several ways to do that:
    #  - give the container a unique name we know, and use that to clean
    #  - maybe use some kind of time frame to identify the container
    #  - warn the user of a potential container leak by using signals

    def stop_remove():
        """Stop then remove the container."""
        if container:
            if container.status == "running":
                container.stop()
                container.wait()
            container.remove()

    atexit.register(stop_remove)

    # split create and start so in case of SIGTERM, container has fewer chances
    # of being None
    container = client.containers.create(
        "xcgd/isort:odoo",
        command=["isort", "--check"],
        volumes={os.environ["PWD"]: {"bind": "/mnt", "mode": "ro"}},
        working_dir="/mnt",
        detach=True,
    )
    container.start()
    result = container.wait()
    atexit.unregister(stop_remove)
    atexit.register(container.remove)
    print(container.logs().decode().rstrip())
    if result.get("Error", None):
        _logger.warning(result.decode())
    container.remove(v=True)
    atexit.unregister(container.remove)
    return result["StatusCode"]


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))
