#!/usr/bin/env python3
# vim: set shiftwidth=4 softtabstop=4:
"""Run a postgresql docker with volume based on project name.
"""
import argparse
import atexit
import logging
import os
import sys
import tempfile
import time
from typing import Callable, Tuple

import docker

from .config import Config
from .docker_client import DockerClient
from .parsing import apply, basic_parser

_logger = logging.getLogger(__name__)

__version__ = "1.0.0"
__date__ = "2020-11-09"
__updated__ = "2020-11-10"

POSTGRES_PASSWORD = "this-could-be-anything"
PSQL = "psql"
SH = "SH"


def docker_run_postgresql(
    project_name: str,
    postgresql_version: str,
    host_pg_port=None,
    stop_at_exit: bool = True,
) -> Tuple[docker.models.containers.Container, Callable, str]:
    """

    :param project_name:
    :param postgresql_version:
    :param host_pg_port: if None, put the socket in a temp directory,
     otherwise publish the port
    :param stop_at_exit: True to stop the container at exit
    :return: container, method to call to stop it, socket directory
    """
    docker_client = DockerClient.client
    pg_repository = "postgres"
    version = "{}-alpine".format(postgresql_version)
    pg_image = "{}:{}".format(pg_repository, version)
    name = "postgresql-{}-{}".format(postgresql_version, project_name)
    # need to pull otherwise the containers.create might fail
    docker_client.images.pull(repository=pg_repository, tag=version)
    pg_data_volume_name = "postgresql_{}-{}".format(
        postgresql_version, project_name
    )
    path: str = None
    env = dict()
    env["POSTGRES_PASSWORD"] = POSTGRES_PASSWORD

    def stop_postgresql(pg_container: docker.models.containers.Container):
        # TODO test if still exists
        #  maybe by looking at pg.status
        # TODO factorized with other methods that start dockers
        try:
            _logger.info("Stopping postgresql %s", pg_container.name)
            pg_container.stop()
            pg_container.wait()
            _logger.info("Removing postgresql container %s", pg_container.name)
            pg_container.remove()
        except docker.errors.NotFound:
            _logger.info(
                "Postgresql container %s already stopped", pg_container.name
            )
        # crash because docker or pg changes the owner of the directory
        # temp_directory.cleanup()

    for container in docker_client.containers.list():
        if name == container.name:
            _logger.info("Postgresql Container already running")
            source = None
            for mount_dict in container.attrs["Mounts"]:
                if mount_dict["Destination"] == "/var/run/postgresql":
                    source = mount_dict["Source"]
            return container, lambda: stop_postgresql(container), source
    volumes = dict()

    volumes[pg_data_volume_name] = {
        "bind": "/var/lib/postgresql/data",
        "mode": "rw",
    }
    port_bindings = dict()
    if host_pg_port:
        port_bindings[5432] = host_pg_port
    else:
        path = tempfile.TemporaryDirectory(
            prefix="odoo_scripts_postgres_socket-"
        ).name
        volumes[path] = {"bind": "/var/run/postgresql", "mode": "rw"}
    _logger.debug("Creating postgresql container")
    pg = docker_client.containers.create(
        pg_image,
        volumes=volumes,
        ports=port_bindings,
        name=name,
        environment=env,
    )

    _logger.info("Starting postgresql container %s", pg.name)
    pg.start()

    def stop_pg():
        return stop_postgresql(pg)

    if stop_at_exit:
        atexit.register(stop_pg)
    _logger.debug("Waiting for postgres to start")
    # give pg the time to start up
    time.sleep(5)
    return pg, stop_pg, path


def __parser() -> argparse.ArgumentParser:
    program_version = __version__
    program_build_date = str(__updated__)
    program_version_message = "%%(prog)s %s (%s)" % (
        program_version,
        program_build_date,
    )
    program_shortdesc = __doc__.split(".")[0]
    program_license = """%s

      Created by Vincent Hatakeyama on %s.
      Copyright 2020 XCG Consulting. All rights reserved.

      Licensed under the MIT License

      Distributed on an "AS IS" basis without warranties
      or conditions of any kind, either express or implied.

    USAGE
    """ % (
        program_shortdesc,
        str(__date__),
    )
    parser = basic_parser(program_license, program_version_message)
    project_path = os.path.realpath(".")
    project_name = os.path.basename(project_path)
    parser.add_argument(
        "--name",
        help="Container name to use [default: %(default)s]",
        default=project_name,
    )
    parser.add_argument(
        "-c",
        "--command",
        help="Command to run. If specified the command will not be interactive"
        ", otherwise it will be interactive.",
        metavar="command",
        default=None,
    )
    exec_group = parser.add_mutually_exclusive_group()
    exec_group.add_argument(
        "--psql",
        help="Exec psql (default)",
        action="store_const",
        const=PSQL,
        dest="exec",
        default=PSQL,
    )
    exec_group.add_argument(
        "--sh",
        help="Exec a shell",
        action="store_const",
        const=SH,
        dest="exec",
    )
    # no simple way to indicate that these options are only used with psql exec
    parser.add_argument("-d", "--database", help="Database name", default=None)
    parser.add_argument("-u", "--user", help="User", default=None)
    return parser


def main(argv=None):
    """Run postgresql docker, and exec a psql into it"""
    parser = __parser()
    namespace = parser.parse_args(argv)
    apply(namespace)
    project_name = namespace.name
    exec_type = namespace.exec
    command = namespace.command
    user = namespace.user
    # get postgres version from config
    c = Config()
    postgresql_version = c.postgresql_version
    pg, stop_pg, path = docker_run_postgresql(project_name, postgresql_version)
    # run an exec command
    docker_command = None
    interactive = not bool(command)
    if exec_type == PSQL:
        if not user:
            user = "postgres"
        docker_command = ["psql", "--user", user]
        if command:
            _logger.info("Exec command in the container")
            docker_command.extend(["-c", command])
        else:
            _logger.info("Exec psql in the container")
        if namespace.database:
            docker_command.append(namespace.database)
        user = "root"
    elif exec_type == SH:
        if not user:
            user = "root"
        docker_command = ["sh"]
        if command:
            _logger.info("Exec shell command in the container")
            docker_command.extend(["-c", command])
        else:
            _logger.info("Exec sh in the container")
    if docker_command:
        return DockerClient.exec(pg, docker_command, user, interactive)


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))
